#!/usr/bin/perl -w
#
# Cisco sample server written in Perl based on the
# perlipc(1) manpage.
# 
# (c) 2002 Javier Fernndez-Sanguino Pea
#
# Distributed under the GNU GPL license version 2 or (at your option)
# any later version.
use IO::Socket;
use Net::hostent;              # for OO version of gethostbyaddr

$MYNAME = "simrouter";
$PORT = 23; # telnet server (obviously should not be in use)

$server = IO::Socket::INET->new( Proto     => 'tcp',
		LocalPort => $PORT,
		Listen    => SOMAXCONN,
		Reuse     => 1);

die "can't setup server: $!" unless $server;
print "[Server $0 accepting clients at $PORT]\n";

while ($client = $server->accept()) {
	$client->autoflush(1);
	$hostinfo = gethostbyaddr($client->peeraddr);
	printf  "[Connect from %s]\n", $hostinfo->name || $client->peerhost;
	my $server_state = "login";
	my $userok = 0; my $passok = 0;
	print $client "Cisco router simulator. Please log in.\n";
	print $client "(you probably know the username/password don't you? :)\n";
	print $client "username: ";
# Command reception BTW, is not perfect, since we do not take into
# account telnet chars... oh well.. this is just an example :(
	while ( $command=<$client>) {
	# TODO: Remove all binary stuff 
	# (that telnet will send in the first connection)
	# Remove CR/LF (mandatory for telnet)
		printf  "[Current state is %s]\n", $server_state ;
		printf  "[Received %s]", $command ;
		chomp $command;
# State machine:
# 0/login - not logged in
# 0/pass - not logged in asking for password
# 1/user - user mode
# 2/enpass - ask for password for enable mode
# 3/enable - enable mode
		for ($server_state) {
			if    (/^login$/)    { 
				$server_state="pass" ;
				$userok = 1 if ( $command =~ /cisco/ ) ;
			}
			elsif (/^pass$/) { 
				if ( $userok == 1 && $command =~ /cisco/ ) {
					$server_state="user" ;
				} else { 
					$server_state="login";
				}
			}
			elsif (/^user$/) { 
# What else can we do here?
				$server_state = "login" if $command =~ /^exit/ ;
				$server_state = "enpass" if $command =~ /^enable/ ;
				print $client "Nothing to show :-)\n" if $command =~ /^show/ ;
			}
			elsif (/^enpass$/) {
				$server_state="enable" if $command =~ /^cisco/ ;
			}
			elsif (/^enable$/) {
# What else can we do here
				$server_state = "user" if  $command =~ /^exit/ ;
			}
		}	
# Now next banner....
		for ($server_state) {
			if    (/^login$/)    { print $client "username: "; }
			elsif (/^pass$/) { print $client "password: "; }
			elsif (/^user$/) { print $client "$MYNAME> "; }
			elsif (/^enpass$/) { print $client "password: "; }
			elsif (/^enable$/) { print $client "$MYNAME# "; }
			else { print "? "; }
		}	
	}
	close $client;
}

